@@ -3,7 +3,7 @@ BEGIN {
   $App::Pocoirc::Status::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $App::Pocoirc::Status::VERSION = '0.40';
+  $App::Pocoirc::Status::VERSION = '0.41';
 }
 
 use strict;
@@ -22,7 +22,7 @@ sub new {
 sub PCI_register {
     my ($self, $irc, %args) = @_;
 
-    $irc->raw_events(1) if $self->{Verbose};
+    $irc->raw_events(1);
     $irc->plugin_register($self, 'SERVER', 'all');
     $irc->plugin_register($self, 'USER', 'all');
     return 1;
@@ -74,7 +74,8 @@ sub _dump {
         return overload::StrVal($arg);
     }
     elsif (defined $arg) {
-        return looks_like_number($arg) ? $arg : "'$arg'";
+        return $arg if looks_like_number($arg);
+        return "'".decode_irc($arg)."'";
     }
     else {
         return 'undef';
@@ -273,9 +274,7 @@ sub S_raw {
 
 sub S_raw_out {
     my ($self, $irc) = splice @_, 0, 2;
-    my $raw = ${ $_[0] };
-    $raw = strip_color($raw);
-    $raw = strip_formatting($raw);
+    my $raw = _normalize(${ $_[0] });
     return PCI_EAT_NONE if !$self->{Verbose};
     $irc->send_event_next('irc_plugin_status', $self, 'debug', ">>> $raw");
     return PCI_EAT_NONE;
